from hysop.deps import sm, np
from hysop.tools.types import check_instance
from hysop.backend.device.codegen.base.opencl_codegen   import OpenClCodeGenerator
from hysop.backend.device.codegen.base.function_codegen import OpenClFunctionCodeGenerator
from hysop.backend.device.codegen.base.variables        import CodegenVariable, CodegenVectorClBuiltin
from hysop.backend.device.codegen.base.utils            import WriteOnceDict, ArgDict
from hysop.backend.device.codegen.base.statistics       import WorkStatistics
from hysop.backend.device.opencl.opencl_types           import OpenClTypeGen, basetype

class ComplexMultiplication(OpenClFunctionCodeGenerator):

    def __init__(self, typegen, ftype, vectorization,
            known_args=None):

        assert vectorization in (1,2,4,8)
        
        ftype = basetype(ftype)
        vtype = typegen.vtype(ftype, 2*vectorization)

        args = ArgDict()
        args['lhs'] = CodegenVectorClBuiltin('lhs', ftype, 2*vectorization, typegen, add_impl_const=True)
        args['rhs'] = CodegenVectorClBuiltin('rhs', ftype, 2*vectorization, typegen, add_impl_const=True)
        
        reqs = WriteOnceDict()

        super(ComplexMultiplication, self).__init__(basename='cmul',
                output=vtype, typegen=typegen, inline=True,
                args=args, known_args=known_args)

        self.update_requirements(reqs)
        
        self.ftype = ftype
        self.vtype = vtype
        self.vectorization = vectorization
        
        self.gencode()
    
    def gencode(self):
        s = self
        vectorization = s.vectorization
        tg = self.typegen
        ftype = s.ftype
        vtype = s.vtype
        
        lhs = s.args['lhs']
        rhs = s.args['rhs']
        
        with s._function_():
            with s._align_() as al:
                init = ''
                for i in xrange(vectorization):
                    real = '{}*{}-{}*{}'.format(lhs[2*i],   rhs[2*i],
                                                lhs[2*i+1], rhs[2*i+1])
                    imag = '{}*{}+{}*{}'.format(lhs[2*i],   rhs[2*i+1],
                                                lhs[2*i+1], rhs[2*i])
                    init += ',\n${},\n${}'.format(real, imag)
                ret = 'return ({})({});'.format(vtype, init[2:])
                al.append(ret)
    

if __name__ == '__main__':

    from hysop.backend.device.codegen.base.test import _test_typegen
    
    tg = _test_typegen('float')
    pf = ComplexMultiplication(tg,'float',4)
    pf.edit()
    pf.test_compile()
