
import sympy as sm
from hysop.backend.device.codegen.symbolic.expr import TypedI, cl_components
from hysop.symbolic.relational import LogicalAND, LogicalOR, LogicalXOR, \
                                      LogicalEQ, LogicalNE, \
                                      LogicalLT, LogicalGT, \
                                      LogicalLE, LogicalGE, \
                                      Mul, Pow, Add
def basetype(fulltype):
    import string
    return fulltype.translate(None, string.digits)

class RelationalTypedExpr(TypedI):
    def __new__(cls, ctype, *args, **kwds):
        obj = super(RelationalTypedExpr, cls).__new__(cls, *args, **kwds)
        obj.ctype = ctype
        return obj

class LogicalTypedExpr(RelationalTypedExpr):
    def __new__(cls, ctype, *args, **kwds):
        assert basetype(ctype) in ('char', 'short', 'int', 'long'), ctype
        return super(LogicalTypedExpr, cls).__new__(cls, ctype, *args, **kwds)

class ArithmeticTypedExpr(RelationalTypedExpr):
    pass

class OpenClLogicalAND(LogicalTypedExpr, LogicalAND):
    pass
class OpenClLogicalOR(LogicalTypedExpr, LogicalOR):
    pass
class OpenClLogicalXOR(LogicalTypedExpr, LogicalXOR):
    pass

class OpenClLogicalEQ(LogicalTypedExpr, LogicalEQ):
    pass
class OpenClLogicalNE(LogicalTypedExpr, LogicalNE):
    pass

class OpenClLogicalLT(LogicalTypedExpr, LogicalLT):
    pass
class OpenClLogicalGT(LogicalTypedExpr, LogicalGT):
    pass

class OpenClLogicalLE(LogicalTypedExpr, LogicalLE):
    pass
class OpenClLogicalGE(LogicalTypedExpr, LogicalGE):
    pass

class OpenClAdd(ArithmeticTypedExpr, Add):
    pass
class OpenClMul(ArithmeticTypedExpr, Mul):
    pass
class OpenClPow(RelationalTypedExpr, Pow):
    def _ccode(self, printer):
        b,e = self.args
        pb = printer._print(b)
        pe = printer._print(e)
        if isinstance(e, TypedI) and e.is_integer:
            fn = 'pown'
        elif isinstance(e, TypedI) and e.positive:
            fn = 'powr'
        else:
            fn = 'pow'
        return '{}({}, {})'.format(fn, pb, pe)

