"""
@file enstrophy.py
Enstrophy solver python backend.
"""
from hysop.tools.decorators import debug
from hysop.core.graph.graph import op_apply
from hysop.operator.base.enstrophy import EnstrophyBase
from hysop.backend.host.host_operator import HostOperator
from hysop.tools.numpywrappers import npw


class PythonEnstrophy(EnstrophyBase, HostOperator):
    """Compute enstrophy of the given vorticity field."""
    @debug
    def __init__(self, **kwds):
        super(PythonEnstrophy, self).__init__(**kwds)
        assert self.mpi_params.size == 1

    @debug
    def setup(self, work):
        super(PythonEnstrophy, self).setup(work=work)
        self.W_buffers    = self.dWin.compute_buffers
        self.WdotW_buffer = self.dWdotW.compute_buffers[0]

    @op_apply
    def apply(self, **kwds):
        W   = self.W_buffers
        WdW = self.WdotW_buffer

        WdW[...] = 0.0
        for Wi in W:
            WdW[...] += (Wi ** 2)

        # Compute enstrophy
        # TODO reduce over all mpi process
        local_enstrophy = self.coeff * npw.sum(WdW, dtype=self.enstrophy.dtype)

        self.enstrophy.value = local_enstrophy
