from hysop.deps import np

from hysop.tools.decorators import debug, profile
from hysop.backend.host.host_operator import HostOperator
from hysop.operator.base.transpose_operator import TransposeOperatorBase
from hysop.core.graph.graph import op_apply

class PythonTranspose(TransposeOperatorBase, HostOperator):
    """
    Inplace and out of place field transposition and permutations in general.
    Uses basic numpy transposition facility.
    """

    @debug
    def __init__(self, **kwds): 
        """Initialize a Transpose operator on the python backend."""
        super(PythonTranspose, self).__init__(**kwds)

    @op_apply
    def apply(self, **kwds):
        """ Transpose in or out of place."""
        super(PythonTranspose,self).apply(**kwds)

        axes = self.axes
        
        if self.is_inplace:
            din, dout, dtmp = self.din, self.dout, self.dtmp.handle.view(np.ndarray)
            assert self.din.dfield is self.dout.dfield
            for i in xrange(din.nb_components):
                dtmp[...] = np.transpose(din.buffers[i], axes=axes)
                dout.buffers[i][...] = dtmp
        else:
            din, dout = self.din, self.dout
            assert self.din.dfield is not self.dout.dfield
            for i in xrange(din.nb_components):
                dout.buffers[i][...] = np.transpose(din.buffers[i], axes=axes)
    
    @classmethod
    def supports_mpi(cls):
        return True
