
from hysop.problem          import Problem
from hysop.operators        import DirectionalAdvection, Poisson
from hysop.numerics.splitting.strang import StrangSplitting

from hysop.domain.box         import Box
from hysop.fields.continuous_field  import Field
from hysop.topology.cartesian_topology import CartesianTopology
from hysop.tools.parameters   import CartesianDiscretization
from hysop.simulation import Simulation

from hysop.methods import *
from hysop.backend.device.kernel_autotuner import AutotunerConfig, AutotunerFlags
# from hysop.constants import callback_profiler 

class TestStrang(object):

    @classmethod
    def setup_class(cls):
        pass

    @classmethod
    def teardown_class(cls):
        pass
    
    def setup_method(self, method):
        pass

    def teardown_method(self, method):
        pass

    def make_fields(self, dim, nscalars):
        # Domain
        box = Box(length=[1.0]*dim, origin=[0.0]*dim)

        ## Fields
        velo = Field(domain=box, formula=None,
                     name='V', is_vector=True)
        if dim==2:
            vorti = Field(domain=box, name='W')
        else:
            vorti = Field(domain=box, formula=None,
                    name='W', is_vector=True)

        scalars = [ Field(domain=box, name='S{}'.format(i))
                           for i in xrange(nscalars)]

        return box, velo, vorti, tuple(scalars)

    def _test_strang_nd(self, dim, n):
        ## Simulation
        simu = Simulation(start=0.0, end=1.0, max_iter=100, time_step=0.1)
        simu.initialize()
        
        ## Domain and continuous fields
        box, velo, vorti, scalars = self.make_fields(dim,2)

        ## CartesianDiscretizations and topologies
        resolution     = (n,)*dim
        dnd            = CartesianDiscretization(resolution=resolution, ghosts=None)
        topo           = CartesianTopology(box,dnd,dim)

        ## Operators
        advec = DirectionalAdvection(
                name='advec',
                velocity = velo,       
                advected_fields = (vorti,)+scalars,
                variables = { velo: topo, vorti: topo,
                              scalars[0]: topo, scalars[1]: topo},
                split_fields=(vorti, scalars)
            )

        poisson = Poisson(name='poisson', 
                velocity=velo, vorticity=vorti, 
                topology=topo)

        ## Graph
        splitting = StrangSplitting(dim=dim, order=StrangOrder.STRANG_FIRST_ORDER)
        splitting.push_operator(advec)

        problem = Problem()
        problem.insert(splitting)
        problem.insert(poisson)
        problem.build()
        
        problem.display()
        problem.finalize()
    
    def test_strang_2d(self, n=33):
        self._test_strang_nd(dim=2, n=n)
    def test_strang_3d(self, n=33):
        self._test_strang_nd(dim=3, n=n)

if __name__ == '__main__':
    test = TestStrang()
    test.setup_class()
    test.test_strang_3d()
    test.teardown_class()
