
from hysop.deps import sm
from hysop.tools.numpywrappers import npw
from hysop.tools.types import check_instance, first_not_None
from hysop.symbolic import Expr, Symbol

class Select(Expr):
    def __new__(cls, a, b, c, *args):
        """ Equivalent to ternary operator: (c ? b : a) """
        obj = super(Select, cls).__new__(cls, a, b, c, *args)
        obj.a = a
        obj.b = b
        obj.c = c
        return obj
    
    def __str__(self):
        return '({} ? {} : {})'.format(self.c, self.b, self.a)
    def __repr__(self):
        return 'Select({}, {}, {})'.format(repr(self.a), repr(self.b), repr(self.c))
    def _sympystr(self, printer):
        return '({} ? {} : {})'.format(printer._print(self.c), 
                                       printer._print(self.b),
                                       printer._print(self.a))


class Expand(Expr):
    """Expand a vector by a given factor.
       v=v.xy => v.xxyy (factor = 2)
    """
    def __new__(cls, expr, factor, *args):
        obj = super(Expand, cls).__new__(cls, expr, factor, *args)
        obj.expr = expr
        obj.factor = factor
        return obj
    def __str__(self):
        return 'Expand({}, {})'.format(self.expr, self.factor)
    def __repr__(self):
        return self.__str__()
    def _sympystr(self, printer):
        return 'Expand({}, {})'.format(printer._print(self.expr), self.factor)

class BroadCast(Expr):
    """
    BroadCast a vector by a given factor.
     v=v.xy => v.xyxy (factor = 2)
    """
    def __new__(cls, expr, factor):
        obj = super(BroadCast, cls).__new__(cls, expr, factor)
        obj.expr = expr
        obj.factor = factor
        return obj
    def __str__(self):
        return 'BroadCast({}, {})'.format(self.expr, self.factor)
    def __repr__(self):
        return self.__str__()
    def _sympystr(self, printer):
        return 'BroadCast({}, {})'.format(printer._print(self.expr), self.factor)

class ApplyStencil(Expr):
    def __new__(cls, expr, stencil):
        obj = super(ApplyStencil, cls).__new__(cls, expr, stencil)
        obj.expr    = expr
        obj.stencil = stencil
        return obj

    def _sympystr(self, printer):
        return 'ApplyStencil({}, {})'.format(printer._print(self.expr), self.stencil.coeffs)

class TimeIntegrate(Expr):
    """
    Represents variable time integration for code generation.

    Parameters
    ----------
    time_integrator: TimeIntegrator
    lhs : Field
    rhs : Expr
    """
    def __new__(cls, time_integrator, lhs, rhs):
        obj = super(TimeIntegrate, cls).__new__(cls, time_integrator, lhs, rhs)
        obj.time_integrator = time_integrator
        obj.lhs = lhs
        obj.rhs = rhs
        return obj

    def __str__(self):
        return '{} = {}({})'.format(self.lhs, self.time_integrator.name(), self.rhs)

    def _ccode(self):
        raise RuntimeError()

