
from hysop.deps import np, time, sys
from contextlib import contextmanager

@contextmanager
def printoptions(*args, **kwargs):
    original = np.get_printoptions()
    np.set_printoptions(*args, **kwargs)
    yield 
    np.set_printoptions(**original)


@contextmanager
def systrace(fn=None):
    __old_trace = sys.gettrace()
    def __trace(frame, event, arg):
        print('{} {}:{}'.format(event, frame.f_code.co_filename, frame.f_lineno))
    fn = fn or __trace
    sys.settrace(fn)
    yield 
    sys.settrace(__old_trace)

class Timer(object):    
    def __enter__(self, factor=1):
        self.start = time.time()
        self.factor = factor
        self.end = None
        self.interval = None
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end = time.time()
        self.interval = (self.end - self.start)*self.factor
        if exc_type:
            raise
