
from hysop.deps import np
from hysop.core.mpi import MPI
from hysop.tools.numerics import get_dtype

def create_sized(nbytes):
    mpi_type = MPI.Datatype.Create_contiguous(MPI.BYTE, nbytes)
    mpi_type.Commit()
    return mpi_type

def iter_mpi_requests(requests):
    """Iterate on MPI events with Waitany."""
    while True:
        idx = MPI.Request.Waitany(requests)
        if (idx == MPI.UNDEFINED):
            break
        yield idx

MPI_HALF = create_sized(2)

def dtype_to_mpi_type(dtype):
    """Try to convert a numpy dtype to a MPI type."""
    if (dtype is None):
        return None
    from mpi4py import MPI
    dtype = get_dtype(dtype)
    mpi_types = {
        np.int8:       MPI.SIGNED_CHAR,
        np.int16:      MPI.SIGNED_SHORT,
        np.int32:      MPI.INT,
        np.int64:      MPI.LONG_LONG,
        np.uint8:      MPI.UNSIGNED_CHAR,
        np.uint16:     MPI.UNSIGNED_SHORT,
        np.uint32:     MPI.UNSIGNED,
        np.uint64:     MPI.UNSIGNED_LONG_LONG,
        np.float16:    MPI_HALF,
        np.float32:    MPI.FLOAT,
        np.float64:    MPI.DOUBLE,
        np.float128:   MPI.LONG_DOUBLE, # (80 bits floats on a 64 bit machine)
        np.complex64:  MPI.COMPLEX,
        np.complex128: MPI.DOUBLE_COMPLEX,
    }
    if hasattr(MPI, 'LONG_DOUBLE_COMPLEX'):
        mpi_types[np.complex256] = MPI.LONG_DOUBLE_COMPLEX

    if (dtype not in mpi_types):
        msg='Unknown dtype {}.'.format(dtype)
        raise NotImplementedError(msg)
    return mpi_types[dtype]
