
import numba as nb
import numpy as np
from hysop.core.arrays.array import Array

def make_numba_signature(*args, **kwds):
    raise_on_cl_array = kwds.pop('raise_on_cl_array', True)
    if kwds:
        msg='Unknown kwds {}.'.forma(kwds.keys())
        raise RuntimeError(kwds)
    dtype_to_ntype = {
            int:        np.int32,
            long:       np.int64,
            float:      np.float64,

            np.int8:    nb.int8,
            np.int16:   nb.int16,
            np.int32:   nb.int32,
            np.int64:   nb.int64,

            np.uint8:   nb.uint8,
            np.uint16:  nb.uint16,
            np.uint32:  nb.uint32,
            np.uint64:  nb.uint64,

            np.float32: nb.float32,
            np.float64: nb.float64,

            np.complex64:  nb.complex64,
            np.complex128: nb.complex128,
    }
    
    sizes = ('m','n','p','q','r','s') + tuple('n{}'.format(i) for i in xrange(10))
    registered_sizes = {}
    def format_shape(*shape):
        res = '('
        for (i,s) in enumerate(shape):
            if s in registered_sizes:
                sc = registered_sizes[s]
            else:
                sc = sizes[len(registered_sizes)]
                registered_sizes[s] = sc
            res += sc
            if (i!=len(shape)-1):
                res+=','
        res += ')'
        return res
    
    numba_args = ()
    numba_layout = ()
    for i,a in enumerate(args):
        from hysop.backend.device.opencl import clArray
        if isinstance(a, Array):
            a = a.handle
        if isinstance(a, clArray.Array):
            # some opencl arrays can be mapped to host
            if raise_on_cl_array:
                msg='Numba signature: Got a cl.Array or hysop.OpenClArray for argument {} (shape={}, dtype={}).'
                msg=msg.format(i, a.shape, a.dtype)
                raise ValueError(msg)
            assert a.dtype.type in dtype_to_ntype, a.dtype.type
            dtype = dtype_to_ntype[a.dtype.type]
            if a.flags.c_contiguous:
                na = nb.types.Array(dtype=dtype, ndim=a.ndim, layout='C')
            elif a.flags.f_contiguous:
                na = nb.types.Array(dtype=dtype, ndim=a.ndim, layout='F')
            else:
                na = nb.types.Array(dtype=dtype, ndim=a.ndim, layout='A')
            numba_layout += (format_shape(*a.shape),)
        elif isinstance(a, np.ndarray):
            assert a.dtype.type in dtype_to_ntype, a.dtype.type
            dtype = dtype_to_ntype[a.dtype.type]
            if a.flags['C_CONTIGUOUS']:
                na = nb.types.Array(dtype=dtype, ndim=a.ndim, layout='C')
            elif a.flags['F_CONTIGUOUS']:
                na = nb.types.Array(dtype=dtype, ndim=a.ndim, layout='F')
            else:
                na = nb.types.Array(dtype=dtype, ndim=a.ndim, layout='A')
            numba_layout += (format_shape(*a.shape),)
        elif isinstance(a, np.dtype):
            na = dtype_to_ntype[a.type]
            numba_layout += ('()',)
        elif isinstance(a, type):
            na = dtype_to_ntype[a]
            numba_layout += ('()',)
        elif type(a) in dtype_to_ntype:
            na = dtype_to_ntype[type(a)]
            numba_layout += ('()',)
        else:
            msg='Uknown argument type {}.'.format(type(a).__mro__)
            raise NotImplementedError(msg)
        
        numba_args += (na,)

    return nb.void(*numba_args), ','.join(numba_layout)
